MAL_ENDPOINT = "https://myanimelist.net/"
NOT_FOUND_TITLE = "404 Not Found - MyAnimeList.net "
NO_BACKGROUND_INFO = "No background information has been added to this title. Help improve our database by adding background information here."

TIMEOUT = 5
